@@ -13,13 +13,14 @@ build_time_vars = {'ABIFLAGS': '',
  'BINDIR': '/usr/pkg/bin',
  'BINLIBDEST': '/usr/pkg/lib/python3.9',
  'BLDLIBRARY': '-L. -lpython3.9',
- 'BLDSHARED': 'gcc -pthread -shared -L. -L/usr/lib -Wl,-R/usr/lib '
+ 'BLDSHARED': 'gcc -pthread -shared -L. -Wl,-zrelro -L/usr/lib -Wl,-R/usr/lib '
               '-L/usr/pkg/lib -Wl,-R/usr/pkg/lib -L/usr/X11R7/lib '
-              '-Wl,-R/usr/X11R7/lib -L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib '
-              '-Wl,-R/usr/pkg/lib -L/usr/X11R7/lib -Wl,-R/usr/X11R7/lib',
+              '-Wl,-R/usr/X11R7/lib -Wl,-zrelro -L/usr/lib -Wl,-R/usr/lib '
+              '-L/usr/pkg/lib -Wl,-R/usr/pkg/lib -L/usr/X11R7/lib '
+              '-Wl,-R/usr/X11R7/lib',
  'BUILDEXE': '',
  'BUILDPYTHON': 'python',
- 'BUILD_GNU_TYPE': 'armv7--netbsdelf-eabihf',
+ 'BUILD_GNU_TYPE': 'powerpc--netbsd',
  'BYTESTR_DEPS': '\\',
  'CC': 'gcc -pthread',
  'CCSHARED': '-fPIC',
@@ -38,30 +39,28 @@ build_time_vars = {'ABIFLAGS': '',
                             '-Werror=implicit-function-declaration '
                             '-fvisibility=hidden',
  'CONFIGURE_CPPFLAGS': '-I/usr/include -I/usr/pkg/include',
- 'CONFIGURE_LDFLAGS': '-L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib '
+ 'CONFIGURE_LDFLAGS': '-Wl,-zrelro -L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib '
                       '-Wl,-R/usr/pkg/lib -L/usr/X11R7/lib '
                       '-Wl,-R/usr/X11R7/lib',
  'CONFIGURE_LDFLAGS_NODIST': '',
  'CONFIG_ARGS': "'--enable-shared' '--with-openssl=/usr' '--with-system-expat' "
                 "'--with-system-ffi' '--without-ensurepip' "
                 "'--without-pymalloc' '--prefix=/usr/pkg' "
-                "'--build=armv7--netbsdelf-eabihf' "
-                "'--host=armv7--netbsdelf-eabihf' '--mandir=/usr/pkg/man' "
-                "'--enable-option-checking=yes' "
-                "'build_alias=armv7--netbsdelf-eabihf' "
-                "'host_alias=armv7--netbsdelf-eabihf' 'CC=gcc' 'CFLAGS=-O2 "
-                "-I/usr/include -I/usr/pkg/include' 'LDFLAGS=-L/usr/lib "
-                '-Wl,-R/usr/lib -L/usr/pkg/lib -Wl,-R/usr/pkg/lib '
-                "-L/usr/X11R7/lib -Wl,-R/usr/X11R7/lib' 'LIBS=' "
-                "'CPPFLAGS=-I/usr/include -I/usr/pkg/include' "
-                "'PKG_CONFIG=/mnt/earmv7hf/pkgsrc/lang/python39/work.earmv7hf/.tools/bin/pkg-config' "
+                "'--build=powerpc--netbsd' '--host=powerpc--netbsd' "
+                "'--mandir=/usr/pkg/man' '--enable-option-checking=yes' "
+                "'build_alias=powerpc--netbsd' 'host_alias=powerpc--netbsd' "
+                "'CC=gcc' 'CFLAGS=-O2 -I/usr/include -I/usr/pkg/include' "
+                "'LDFLAGS=-Wl,-zrelro -L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib "
+                "-Wl,-R/usr/pkg/lib -L/usr/X11R7/lib -Wl,-R/usr/X11R7/lib' "
+                "'LIBS=' 'CPPFLAGS=-I/usr/include -I/usr/pkg/include' "
+                "'PKG_CONFIG=/tmp/pkgbuild/lang/python39/work/.tools/bin/pkg-config' "
                 "'PKG_CONFIG_PATH=' "
-                "'PKG_CONFIG_LIBDIR=/mnt/earmv7hf/pkgsrc/lang/python39/work.earmv7hf/.buildlink/lib/pkgconfig:/mnt/earmv7hf/pkgsrc/lang/python39/work.earmv7hf/.buildlink/share/pkgconfig'",
+                "'PKG_CONFIG_LIBDIR=/tmp/pkgbuild/lang/python39/work/.buildlink/lib/pkgconfig:/tmp/pkgbuild/lang/python39/work/.buildlink/share/pkgconfig'",
  'CONFINCLUDEDIR': '/usr/pkg/include',
  'CONFINCLUDEPY': '/usr/pkg/include/python3.9',
  'COREPYTHONPATH': '',
- 'COVERAGE_INFO': '/mnt/earmv7hf/pkgsrc/lang/python39/work.earmv7hf/Python-3.9.21/coverage.info',
- 'COVERAGE_REPORT': '/mnt/earmv7hf/pkgsrc/lang/python39/work.earmv7hf/Python-3.9.21/lcov-report',
+ 'COVERAGE_INFO': '/tmp/pkgbuild/lang/python39/work/Python-3.9.23/coverage.info',
+ 'COVERAGE_REPORT': '/tmp/pkgbuild/lang/python39/work/Python-3.9.23/lcov-report',
  'COVERAGE_REPORT_OPTIONS': '--no-branch-coverage --title "CPython lcov '
                             'report"',
  'CPPFLAGS': '-I. -I./Include -I/usr/include -I/usr/pkg/include -I/usr/include '
@@ -82,8 +81,8 @@ build_time_vars = {'ABIFLAGS': '',
  'DLINCLDIR': '.',
  'DLLLIBRARY': '',
  'DOUBLE_IS_ARM_MIXED_ENDIAN_IEEE754': 0,
- 'DOUBLE_IS_BIG_ENDIAN_IEEE754': 0,
- 'DOUBLE_IS_LITTLE_ENDIAN_IEEE754': 1,
+ 'DOUBLE_IS_BIG_ENDIAN_IEEE754': 1,
+ 'DOUBLE_IS_LITTLE_ENDIAN_IEEE754': 0,
  'DTRACE': '',
  'DTRACE_DEPS': '\\',
  'DTRACE_HEADERS': '',
@@ -99,7 +98,7 @@ build_time_vars = {'ABIFLAGS': '',
  'EXTRA_CFLAGS': '',
  'EXT_SUFFIX': '.so',
  'FILEMODE': 644,
- 'FLOAT_WORDS_BIGENDIAN': 0,
+ 'FLOAT_WORDS_BIGENDIAN': 1,
  'FLOCK_NEEDS_LIBBSD': 0,
  'GETPGRP_HAVE_ARG': 0,
  'GITBRANCH': '',
@@ -314,7 +313,7 @@ build_time_vars = {'ABIFLAGS': '',
  'HAVE_MADVISE': 1,
  'HAVE_MAKEDEV': 1,
  'HAVE_MBRTOWC': 1,
- 'HAVE_MEMFD_CREATE': 1,
+ 'HAVE_MEMFD_CREATE': 0,
  'HAVE_MEMORY_H': 1,
  'HAVE_MEMRCHR': 1,
  'HAVE_MKDIRAT': 1,
@@ -535,31 +534,32 @@ build_time_vars = {'ABIFLAGS': '',
  'HAVE_X509_VERIFY_PARAM_SET1_HOST': 1,
  'HAVE_ZLIB_COPY': 1,
  'HAVE__GETPTY': 0,
- 'HOST_GNU_TYPE': 'armv7--netbsdelf-eabihf',
+ 'HOST_GNU_TYPE': 'powerpc--netbsd',
  'INCLDIRSTOMAKE': '/usr/pkg/include /usr/pkg/include '
                    '/usr/pkg/include/python3.9 /usr/pkg/include/python3.9',
  'INCLUDEDIR': '/usr/pkg/include',
  'INCLUDEPY': '/usr/pkg/include/python3.9',
- 'INSTALL': '/usr/bin/install -c -o root -g wheel',
- 'INSTALL_DATA': '/usr/bin/install -c -o root -g wheel -m 644',
- 'INSTALL_PROGRAM': '/usr/bin/install -c -s -o root -g wheel -m 755',
- 'INSTALL_SCRIPT': '/usr/bin/install -c -o root -g wheel -m 755',
- 'INSTALL_SHARED': '/usr/bin/install -c -o root -g wheel -m 755',
+ 'INSTALL': '/usr/bin/install -c -o pbulk -g users',
+ 'INSTALL_DATA': '/usr/bin/install -c -o pbulk -g users -m 644',
+ 'INSTALL_PROGRAM': '/usr/bin/install -c -s -o pbulk -g users -m 755',
+ 'INSTALL_SCRIPT': '/usr/bin/install -c -o pbulk -g users -m 755',
+ 'INSTALL_SHARED': '/usr/bin/install -c -o pbulk -g users -m 755',
  'INSTSONAME': 'libpython3.9.so.1.0',
  'IO_H': 'Modules/_io/_iomodule.h',
  'IO_OBJS': '\\',
  'LDCXXSHARED': 'c++ -pthread -shared',
- 'LDFLAGS': '-L. -L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib -Wl,-R/usr/pkg/lib '
-            '-L/usr/X11R7/lib -Wl,-R/usr/X11R7/lib -L/usr/lib -Wl,-R/usr/lib '
-            '-L/usr/pkg/lib -Wl,-R/usr/pkg/lib -L/usr/X11R7/lib '
-            '-Wl,-R/usr/X11R7/lib',
+ 'LDFLAGS': '-L. -Wl,-zrelro -L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib '
+            '-Wl,-R/usr/pkg/lib -L/usr/X11R7/lib -Wl,-R/usr/X11R7/lib '
+            '-Wl,-zrelro -L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib '
+            '-Wl,-R/usr/pkg/lib -L/usr/X11R7/lib -Wl,-R/usr/X11R7/lib',
  'LDFLAGS_NODIST': '',
  'LDLIBRARY': 'libpython3.9.so',
  'LDLIBRARYDIR': '',
- 'LDSHARED': 'gcc -pthread -shared -L. -L/usr/lib -Wl,-R/usr/lib '
+ 'LDSHARED': 'gcc -pthread -shared -L. -Wl,-zrelro -L/usr/lib -Wl,-R/usr/lib '
              '-L/usr/pkg/lib -Wl,-R/usr/pkg/lib -L/usr/X11R7/lib '
-             '-Wl,-R/usr/X11R7/lib -L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib '
-             '-Wl,-R/usr/pkg/lib -L/usr/X11R7/lib -Wl,-R/usr/X11R7/lib',
+             '-Wl,-R/usr/X11R7/lib -Wl,-zrelro -L/usr/lib -Wl,-R/usr/lib '
+             '-L/usr/pkg/lib -Wl,-R/usr/pkg/lib -L/usr/X11R7/lib '
+             '-Wl,-R/usr/X11R7/lib',
  'LDVERSION': '3.9',
  'LIBC': '',
  'LIBDEST': '/usr/pkg/lib/python3.9',
@@ -684,17 +684,17 @@ build_time_vars = {'ABIFLAGS': '',
                    '-I./Include/internal -I. -I./Include -I/usr/include '
                    '-I/usr/pkg/include -I/usr/include -I/usr/pkg/include -fPIC '
                    '-DPy_BUILD_CORE',
- 'PY_CORE_LDFLAGS': '-L. -L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib '
+ 'PY_CORE_LDFLAGS': '-L. -Wl,-zrelro -L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib '
                     '-Wl,-R/usr/pkg/lib -L/usr/X11R7/lib -Wl,-R/usr/X11R7/lib '
-                    '-L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib '
+                    '-Wl,-zrelro -L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib '
                     '-Wl,-R/usr/pkg/lib -L/usr/X11R7/lib -Wl,-R/usr/X11R7/lib',
  'PY_CPPFLAGS': '-I. -I./Include -I/usr/include -I/usr/pkg/include '
                 '-I/usr/include -I/usr/pkg/include',
  'PY_FORMAT_SIZE_T': '"z"',
- 'PY_LDFLAGS': '-L. -L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib '
+ 'PY_LDFLAGS': '-L. -Wl,-zrelro -L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib '
                '-Wl,-R/usr/pkg/lib -L/usr/X11R7/lib -Wl,-R/usr/X11R7/lib '
-               '-L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib -Wl,-R/usr/pkg/lib '
-               '-L/usr/X11R7/lib -Wl,-R/usr/X11R7/lib',
+               '-Wl,-zrelro -L/usr/lib -Wl,-R/usr/lib -L/usr/pkg/lib '
+               '-Wl,-R/usr/pkg/lib -L/usr/X11R7/lib -Wl,-R/usr/X11R7/lib',
  'PY_LDFLAGS_NODIST': '',
  'PY_SSL_DEFAULT_CIPHERS': 1,
  'PY_SSL_DEFAULT_CIPHER_STRING': 0,
@@ -716,7 +716,7 @@ build_time_vars = {'ABIFLAGS': '',
  'READELF': 'readelf',
  'RESSRCDIR': 'Mac/Resources/framework',
  'RETSIGTYPE': 'void',
- 'RUNSHARED': 'LD_LIBRARY_PATH=/mnt/earmv7hf/pkgsrc/lang/python39/work.earmv7hf/Python-3.9.21',
+ 'RUNSHARED': 'LD_LIBRARY_PATH=/tmp/pkgbuild/lang/python39/work/Python-3.9.23',
  'SCRIPTDIR': '/usr/pkg/lib',
  'SETPGRP_HAVE_ARG': 1,
  'SHELL': '/bin/sh',
@@ -757,10 +757,10 @@ build_time_vars = {'ABIFLAGS': '',
  'TCLTK_LIBS': '',
  'TESTOPTS': '',
  'TESTPATH': '',
- 'TESTPYTHON': 'LD_LIBRARY_PATH=/mnt/earmv7hf/pkgsrc/lang/python39/work.earmv7hf/Python-3.9.21 '
+ 'TESTPYTHON': 'LD_LIBRARY_PATH=/tmp/pkgbuild/lang/python39/work/Python-3.9.23 '
                './python',
  'TESTPYTHONOPTS': '',
- 'TESTRUNNER': 'LD_LIBRARY_PATH=/mnt/earmv7hf/pkgsrc/lang/python39/work.earmv7hf/Python-3.9.21 '
+ 'TESTRUNNER': 'LD_LIBRARY_PATH=/tmp/pkgbuild/lang/python39/work/Python-3.9.23 '
                './python ./Tools/scripts/run_tests.py',
  'TESTTIMEOUT': 1200,
  'TIMEMODULE_LIB': 0,
@@ -783,8 +783,8 @@ build_time_vars = {'ABIFLAGS': '',
  'WITH_VALGRIND': 0,
  'X87_DOUBLE_ROUNDING': 0,
  'XMLLIBSUBDIRS': 'xml xml/dom xml/etree xml/parsers xml/sax',
- 'abs_builddir': '/mnt/earmv7hf/pkgsrc/lang/python39/work.earmv7hf/Python-3.9.21',
- 'abs_srcdir': '/mnt/earmv7hf/pkgsrc/lang/python39/work.earmv7hf/Python-3.9.21',
+ 'abs_builddir': '/tmp/pkgbuild/lang/python39/work/Python-3.9.23',
+ 'abs_srcdir': '/tmp/pkgbuild/lang/python39/work/Python-3.9.23',
  'datarootdir': '/usr/pkg/share',
  'exec_prefix': '/usr/pkg',
  'prefix': '/usr/pkg',
