@@ -49,8 +49,6 @@ import os
 from concurrent.futures import _base
 import queue
 import multiprocessing as mp
-# This import is required to load the multiprocessing.connection submodule
-# so that it can be accessed later as `mp.connection`
 import multiprocessing.connection
 from multiprocessing.queues import Queue
 import threading
@@ -58,7 +56,7 @@ import weakref
 from functools import partial
 import itertools
 import sys
-from traceback import format_exception
+import traceback
 
 
 _threads_wakeups = weakref.WeakKeyDictionary()
@@ -68,31 +66,22 @@ _global_shutdown = False
 class _ThreadWakeup:
     def __init__(self):
         self._closed = False
-        self._lock = threading.Lock()
         self._reader, self._writer = mp.Pipe(duplex=False)
 
     def close(self):
-        # Please note that we do not take the self._lock when
-        # calling clear() (to avoid deadlocking) so this method can
-        # only be called safely from the same thread as all calls to
-        # clear() even if you hold the lock. Otherwise we
-        # might try to read from the closed pipe.
-        with self._lock:
-            if not self._closed:
-                self._closed = True
-                self._writer.close()
-                self._reader.close()
+        if not self._closed:
+            self._closed = True
+            self._writer.close()
+            self._reader.close()
 
     def wakeup(self):
-        with self._lock:
-            if not self._closed:
-                self._writer.send_bytes(b"")
+        if not self._closed:
+            self._writer.send_bytes(b"")
 
     def clear(self):
-        if self._closed:
-            raise RuntimeError('operation on closed _ThreadWakeup')
-        while self._reader.poll():
-            self._reader.recv_bytes()
+        if not self._closed:
+            while self._reader.poll():
+                self._reader.recv_bytes()
 
 
 def _python_exit():
@@ -134,7 +123,8 @@ class _RemoteTraceback(Exception):
 
 class _ExceptionWithTraceback:
     def __init__(self, exc, tb):
-        tb = ''.join(format_exception(type(exc), exc, tb))
+        tb = traceback.format_exception(type(exc), exc, tb)
+        tb = ''.join(tb)
         self.exc = exc
         # Traceback object needs to be garbage-collected as its frames
         # contain references to all the objects in the exception scope
@@ -155,11 +145,10 @@ class _WorkItem(object):
         self.kwargs = kwargs
 
 class _ResultItem(object):
-    def __init__(self, work_id, exception=None, result=None, exit_pid=None):
+    def __init__(self, work_id, exception=None, result=None):
         self.work_id = work_id
         self.exception = exception
         self.result = result
-        self.exit_pid = exit_pid
 
 class _CallItem(object):
     def __init__(self, work_id, fn, args, kwargs):
@@ -171,17 +160,20 @@ class _CallItem(object):
 
 class _SafeQueue(Queue):
     """Safe Queue set exception to the future object linked to a job"""
-    def __init__(self, max_size=0, *, ctx, pending_work_items, thread_wakeup):
+    def __init__(self, max_size=0, *, ctx, pending_work_items, shutdown_lock,
+                 thread_wakeup):
         self.pending_work_items = pending_work_items
+        self.shutdown_lock = shutdown_lock
         self.thread_wakeup = thread_wakeup
         super().__init__(max_size, ctx=ctx)
 
     def _on_queue_feeder_error(self, e, obj):
         if isinstance(obj, _CallItem):
-            tb = format_exception(type(e), e, e.__traceback__)
+            tb = traceback.format_exception(type(e), e, e.__traceback__)
             e.__cause__ = _RemoteTraceback('\n"""\n{}"""'.format(''.join(tb)))
             work_item = self.pending_work_items.pop(obj.work_id, None)
-            self.thread_wakeup.wakeup()
+            with self.shutdown_lock:
+                self.thread_wakeup.wakeup()
             # work_item can be None if another process terminated. In this
             # case, the executor_manager_thread fails all work_items
             # with BrokenProcessPool
@@ -213,19 +205,17 @@ def _process_chunk(fn, chunk):
     return [fn(*args) for args in chunk]
 
 
-def _sendback_result(result_queue, work_id, result=None, exception=None,
-                     exit_pid=None):
+def _sendback_result(result_queue, work_id, result=None, exception=None):
     """Safely send back the given result or exception"""
     try:
         result_queue.put(_ResultItem(work_id, result=result,
-                                     exception=exception, exit_pid=exit_pid))
+                                     exception=exception))
     except BaseException as e:
         exc = _ExceptionWithTraceback(e, e.__traceback__)
-        result_queue.put(_ResultItem(work_id, exception=exc,
-                                     exit_pid=exit_pid))
+        result_queue.put(_ResultItem(work_id, exception=exc))
 
 
-def _process_worker(call_queue, result_queue, initializer, initargs, max_tasks=None):
+def _process_worker(call_queue, result_queue, initializer, initargs):
     """Evaluates calls from call_queue and places the results in result_queue.
 
     This worker is run in a separate process.
@@ -246,38 +236,25 @@ def _process_worker(call_queue, result_queue, initializer, initargs, max_tasks=N
             # The parent will notice that the process stopped and
             # mark the pool broken
             return
-    num_tasks = 0
-    exit_pid = None
     while True:
         call_item = call_queue.get(block=True)
         if call_item is None:
             # Wake up queue management thread
             result_queue.put(os.getpid())
             return
-
-        if max_tasks is not None:
-            num_tasks += 1
-            if num_tasks >= max_tasks:
-                exit_pid = os.getpid()
-
         try:
             r = call_item.fn(*call_item.args, **call_item.kwargs)
         except BaseException as e:
             exc = _ExceptionWithTraceback(e, e.__traceback__)
-            _sendback_result(result_queue, call_item.work_id, exception=exc,
-                             exit_pid=exit_pid)
+            _sendback_result(result_queue, call_item.work_id, exception=exc)
         else:
-            _sendback_result(result_queue, call_item.work_id, result=r,
-                             exit_pid=exit_pid)
+            _sendback_result(result_queue, call_item.work_id, result=r)
             del r
 
         # Liberate the resource as soon as possible, to avoid holding onto
         # open files or shared memory that is not needed anymore
         del call_item
 
-        if exit_pid is not None:
-            return
-
 
 class _ExecutorManagerThread(threading.Thread):
     """Manages the communication between this process and the worker processes.
@@ -306,10 +283,12 @@ class _ExecutorManagerThread(threading.Thread):
         # will wake up the queue management thread so that it can terminate
         # if there is no pending work item.
         def weakref_cb(_,
-                       thread_wakeup=self.thread_wakeup):
+                       thread_wakeup=self.thread_wakeup,
+                       shutdown_lock=self.shutdown_lock):
             mp.util.debug('Executor collected: triggering callback for'
                           ' QueueManager wakeup')
-            thread_wakeup.wakeup()
+            with shutdown_lock:
+                thread_wakeup.wakeup()
 
         self.executor_reference = weakref.ref(executor, weakref_cb)
 
@@ -326,10 +305,6 @@ class _ExecutorManagerThread(threading.Thread):
         # A queue.Queue of work ids e.g. Queue([5, 6, ...]).
         self.work_ids_queue = executor._work_ids
 
-        # Maximum number of tasks a worker process can execute before
-        # exiting safely
-        self.max_tasks_per_child = executor._max_tasks_per_child
-
         # A dict mapping work ids to _WorkItems e.g.
         #     {5: <_WorkItem...>, 6: <_WorkItem...>, ...}
         self.pending_work_items = executor._pending_work_items
@@ -340,14 +315,7 @@ class _ExecutorManagerThread(threading.Thread):
         # Main loop for the executor manager thread.
 
         while True:
-            # gh-109047: During Python finalization, self.call_queue.put()
-            # creation of a thread can fail with RuntimeError.
-            try:
-                self.add_call_item_to_queue()
-            except BaseException as exc:
-                cause = format_exception(exc)
-                self.terminate_broken(cause)
-                return
+            self.add_call_item_to_queue()
 
             result_item, is_broken, cause = self.wait_result_broken_or_wakeup()
 
@@ -356,32 +324,19 @@ class _ExecutorManagerThread(threading.Thread):
                 return
             if result_item is not None:
                 self.process_result_item(result_item)
-
-                process_exited = result_item.exit_pid is not None
-                if process_exited:
-                    p = self.processes.pop(result_item.exit_pid)
-                    p.join()
-
                 # Delete reference to result_item to avoid keeping references
                 # while waiting on new results.
                 del result_item
 
-                if executor := self.executor_reference():
-                    if process_exited:
-                        with self.shutdown_lock:
-                            executor._adjust_process_count()
-                    else:
-                        executor._idle_worker_semaphore.release()
-                    del executor
+                # attempt to increment idle process count
+                executor = self.executor_reference()
+                if executor is not None:
+                    executor._idle_worker_semaphore.release()
+                del executor
 
             if self.is_shutting_down():
                 self.flag_executor_shutting_down()
 
-                # When only canceled futures remain in pending_work_items, our
-                # next call to wait_result_broken_or_wakeup would hang forever.
-                # This makes sure we have some running futures or none at all.
-                self.add_call_item_to_queue()
-
                 # Since no new work items can be added, it is safe to shutdown
                 # this thread if there are no pending work items.
                 if not self.pending_work_items:
@@ -431,13 +386,14 @@ class _ExecutorManagerThread(threading.Thread):
             try:
                 result_item = result_reader.recv()
                 is_broken = False
-            except BaseException as exc:
-                cause = format_exception(exc)
+            except BaseException as e:
+                cause = traceback.format_exception(type(e), e, e.__traceback__)
 
         elif wakeup_reader in ready:
             is_broken = False
 
-        self.thread_wakeup.clear()
+        with self.shutdown_lock:
+            self.thread_wakeup.clear()
 
         return result_item, is_broken, cause
 
@@ -474,7 +430,7 @@ class _ExecutorManagerThread(threading.Thread):
         return (_global_shutdown or executor is None
                 or executor._shutdown_thread)
 
-    def _terminate_broken(self, cause):
+    def terminate_broken(self, cause):
         # Terminate the executor because it is in a broken state. The cause
         # argument can be used to display more information on the error that
         # lead the executor into becoming broken.
@@ -499,14 +455,7 @@ class _ExecutorManagerThread(threading.Thread):
 
         # Mark pending tasks as failed.
         for work_id, work_item in self.pending_work_items.items():
-            try:
-                work_item.future.set_exception(bpe)
-            except _base.InvalidStateError:
-                # set_exception() fails if the future is cancelled: ignore it.
-                # Trying to check if the future is cancelled before calling
-                # set_exception() would leave a race condition if the future is
-                # cancelled between the check and set_exception().
-                pass
+            work_item.future.set_exception(bpe)
             # Delete references to object. See issue16284
             del work_item
         self.pending_work_items.clear()
@@ -516,14 +465,8 @@ class _ExecutorManagerThread(threading.Thread):
         for p in self.processes.values():
             p.terminate()
 
-        self.call_queue._terminate_broken()
-
         # clean up resources
-        self._join_executor_internals(broken=True)
-
-    def terminate_broken(self, cause):
-        with self.shutdown_lock:
-            self._terminate_broken(cause)
+        self.join_executor_internals()
 
     def flag_executor_shutting_down(self):
         # Flag the executor as shutting down and cancel remaining tasks if
@@ -566,24 +509,15 @@ class _ExecutorManagerThread(threading.Thread):
                     break
 
     def join_executor_internals(self):
-        with self.shutdown_lock:
-            self._join_executor_internals()
-
-    def _join_executor_internals(self, broken=False):
-        # If broken, call_queue was closed and so can no longer be used.
-        if not broken:
-            self.shutdown_workers()
-
+        self.shutdown_workers()
         # Release the queue's resources as soon as possible.
         self.call_queue.close()
         self.call_queue.join_thread()
-        self.thread_wakeup.close()
-
+        with self.shutdown_lock:
+            self.thread_wakeup.close()
         # If .join() is not called on the created processes then
         # some ctx.Queue methods may deadlock on Mac OS X.
         for p in self.processes.values():
-            if broken:
-                p.terminate()
             p.join()
 
     def get_n_children_alive(self):
@@ -601,14 +535,6 @@ def _check_system_limits():
         if _system_limited:
             raise NotImplementedError(_system_limited)
     _system_limits_checked = True
-    try:
-        import multiprocessing.synchronize
-    except ImportError:
-        _system_limited = (
-            "This Python build lacks multiprocessing.synchronize, usually due "
-            "to named semaphores being unavailable on this platform."
-        )
-        raise NotImplementedError(_system_limited)
     try:
         nsems_max = os.sysconf("SC_SEM_NSEMS_MAX")
     except (AttributeError, ValueError):
@@ -648,24 +574,17 @@ class BrokenProcessPool(_base.BrokenExecutor):
 
 class ProcessPoolExecutor(_base.Executor):
     def __init__(self, max_workers=None, mp_context=None,
-                 initializer=None, initargs=(), *, max_tasks_per_child=None):
+                 initializer=None, initargs=()):
         """Initializes a new ProcessPoolExecutor instance.
 
         Args:
             max_workers: The maximum number of processes that can be used to
                 execute the given calls. If None or not given then as many
                 worker processes will be created as the machine has processors.
-            mp_context: A multiprocessing context to launch the workers created
-                using the multiprocessing.get_context('start method') API. This
+            mp_context: A multiprocessing context to launch the workers. This
                 object should provide SimpleQueue, Queue and Process.
             initializer: A callable used to initialize worker processes.
             initargs: A tuple of arguments to pass to the initializer.
-            max_tasks_per_child: The maximum number of tasks a worker process
-                can complete before it will exit and be replaced with a fresh
-                worker process. The default of None means worker process will
-                live as long as the executor. Requires a non-'fork' mp_context
-                start method. When given, we default to using 'spawn' if no
-                mp_context is supplied.
         """
         _check_system_limits()
 
@@ -685,10 +604,7 @@ class ProcessPoolExecutor(_base.Executor):
             self._max_workers = max_workers
 
         if mp_context is None:
-            if max_tasks_per_child is not None:
-                mp_context = mp.get_context("spawn")
-            else:
-                mp_context = mp.get_context()
+            mp_context = mp.get_context()
         self._mp_context = mp_context
 
         # https://github.com/python/cpython/issues/90622
@@ -700,18 +616,6 @@ class ProcessPoolExecutor(_base.Executor):
         self._initializer = initializer
         self._initargs = initargs
 
-        if max_tasks_per_child is not None:
-            if not isinstance(max_tasks_per_child, int):
-                raise TypeError("max_tasks_per_child must be an integer")
-            elif max_tasks_per_child <= 0:
-                raise ValueError("max_tasks_per_child must be >= 1")
-            if self._mp_context.get_start_method(allow_none=False) == "fork":
-                # https://github.com/python/cpython/issues/90622
-                raise ValueError("max_tasks_per_child is incompatible with"
-                                 " the 'fork' multiprocessing start method;"
-                                 " supply a different mp_context.")
-        self._max_tasks_per_child = max_tasks_per_child
-
         # Management thread
         self._executor_manager_thread = None
 
@@ -734,9 +638,7 @@ class ProcessPoolExecutor(_base.Executor):
         # as it could result in a deadlock if a worker process dies with the
         # _result_queue write lock still acquired.
         #
-        # Care must be taken to only call clear and close from the
-        # executor_manager_thread, since _ThreadWakeup.clear() is not protected
-        # by a lock.
+        # _shutdown_lock must be locked to access _ThreadWakeup.
         self._executor_manager_thread_wakeup = _ThreadWakeup()
 
         # Create communication channels for the executor
@@ -747,6 +649,7 @@ class ProcessPoolExecutor(_base.Executor):
         self._call_queue = _SafeQueue(
             max_size=queue_size, ctx=self._mp_context,
             pending_work_items=self._pending_work_items,
+            shutdown_lock=self._shutdown_lock,
             thread_wakeup=self._executor_manager_thread_wakeup)
         # Killed worker processes can produce spurious "broken pipe"
         # tracebacks in the queue's own worker thread. But we detect killed
@@ -794,8 +697,7 @@ class ProcessPoolExecutor(_base.Executor):
             args=(self._call_queue,
                   self._result_queue,
                   self._initializer,
-                  self._initargs,
-                  self._max_tasks_per_child))
+                  self._initargs))
         p.start()
         self._processes[p.pid] = p
 
