@@ -50,10 +50,10 @@ escapesre = re.compile(r'[\\"]')
 
 
 def _has_surrogates(s):
-    """Return True if s may contain surrogate-escaped binary data."""
+    """Return True if s contains surrogate-escaped binary data."""
     # This check is based on the fact that unless there are surrogates, utf8
     # (Python's default encoding) can encode any string.  This is the fastest
-    # way to check for surrogates, see bpo-11454 (moved to gh-55663) for timings.
+    # way to check for surrogates, see issue 11454 for timings.
     try:
         s.encode()
         return False
@@ -311,10 +311,7 @@ def make_msgid(idstring=None, domain=None):
 
 
 def parsedate_to_datetime(data):
-    parsed_date_tz = _parsedate_tz(data)
-    if parsed_date_tz is None:
-        raise ValueError('Invalid date value or format "%s"' % str(data))
-    *dtuple, tz = parsed_date_tz
+    *dtuple, tz = _parsedate_tz(data)
     if tz is None:
         return datetime.datetime(*dtuple[:6])
     return datetime.datetime(*dtuple[:6],
