@@ -1,4 +1,4 @@
-#!/usr/pkg/bin/python3.7
+#!/usr/pkg/bin/python3.9
 """       turtle-example-suite:
 
             tdemo_paint.py
@@ -7,7 +7,7 @@ A simple  event-driven paint program
 
 - left mouse button moves turtle
 - middle mouse button changes color
-- right mouse button toogles betweem pen up
+- right mouse button toggles between pen up
 (no line drawn when the turtle moves) and
 pen down (line is drawn). If pen up follows
 at least two pen-down moves, the polygon that
